#ifndef PE_COMMON_H
#define PE_COMMON_H 1

/*
 * Purpose of this file:
 *
 *   function declarations, portability code, include files or macros used in
 *   nearly every other place of the project.
 */

/* The first thing todo is to include the config.h generated by
 * autoconf/configure, if it exists */
#include <common/config.h>

/* Include some often used header files 
#include <stdio.h>
#include <sys/types.h>

#if STDC_HEADERS
#  include <stdlib.h>
#  include <string.h>
#elif HAVE_STRINGS_H
#  include <strings.h>
#endif 

#if HAVE_UNISTD_H
#  include <unistd.h>
#endif

#if HAVE_ERRNO_H
#  include <errno.h>
#endif 
#ifndef errno
// Some systems #define this! 
extern int errno;
#endif

// provide a replacement for bzero if it is not available 
#if !HAVE_BZERO && HAVE_MEMSET
#  define bzero(buf, bytes) ((void) memset(buf, 0, bytes))
#endif
*/


/* if including C source from within C++ code, the function definitions must be
 * enclosed in "extern "C"" { ... }, so provide a often used macro */

#ifdef __cplusplus
#  define BEGIN_C_DECLS extern "C" {
#  define END_C_DECLS   }
#else /* !__cplusplus */
#  define BEGIN_C_DECLS
#  define END_C_DECLS
#endif /* __cplusplus */

/* for readability of the code, we provide two general defines for exitcodes */
#ifndef EXIT_SUCCESS
#  define EXIT_SUCCESS 0
#  define EXIT_FAILURE 1
#endif

/* Logging */
#if ENABLE_LOGGING
#include <log4cxx/logger.h>

#ifdef LOG4CXX_STR
#  define LOG4CXX_VERSION 1001
#else /* HAVE_LOG4CXX_LOGSTRING_H */
/* version 0.9.7 of log4cxx do not have logstring.h
   we have to define LogString and LOG4CXX_STR
*/
#  define LOG4CXX_STR(a) _T(a)
namespace log4cxx {
   typedef char logchar;
   typedef std::basic_string<logchar> LogString;
}
#  define LOG4CXX_VERSION 907
#endif /* HAVE_LOG4CXX_LOGSTRING_H */

#define LDECLARE_LOGGER(logger)           ::log4cxx::LoggerPtr logger
#define LDEFINE_LOGGER(logger, hierarchy) ::log4cxx::LoggerPtr LINIT_LOGGER(logger, hierarchy)
#define LINIT_LOGGER(logger, hierarchy)   logger(::log4cxx::Logger::getLogger(hierarchy))

#define DECLARE_LOGGER()         LDECLARE_LOGGER(logger)
#define DEFINE_LOGGER(hierarchy) LDEFINE_LOGGER(logger, hierarchy)
#define INIT_LOGGER(hierarchy)   LINIT_LOGGER(logger, hierarchy)

#define LLOG_DEBUG(logger, msg) LOG4CXX_DEBUG(logger, msg)
#define LLOG_INFO(logger, msg)  LOG4CXX_INFO(logger, msg)
#define LLOG_WARN(logger, msg)  LOG4CXX_WARN(logger, msg)
#define LLOG_ERROR(logger, msg) LOG4CXX_ERROR(logger, msg)
#define LLOG_FATAL(logger, msg) LOG4CXX_FATAL(logger, msg)

#define LOG_DEBUG(msg) LLOG_DEBUG(logger, msg)
#define LOG_INFO(msg)  LLOG_INFO(logger, msg) 
#define LOG_WARN(msg)  LLOG_WARN(logger, msg) 
#define LOG_ERROR(msg) LLOG_ERROR(logger, msg)
#define LOG_FATAL(msg) LLOG_FATAL(logger, msg)

#else

#define LDECLARE_LOGGER(logger)   void* __unused_##logger
#define LDEFINE_LOGGER(logger, h)
#define LINIT_LOGGER(logger, h)   __unused_##logger(0)

#define DECLARE_LOGGER()          LDECLARE_LOGGER(logger)
#define DEFINE_LOGGER(hierarchy)  LDEFINE_LOGGER(logger, hierarchy)
#define INIT_LOGGER(hierarchy)    LINIT_LOGGER(logger, hierarchy)

#define LOG_DEBUG(msg) 
#define LOG_INFO(msg)  
#define LOG_WARN(msg)  
#define LOG_ERROR(msg) 
#define LOG_FATAL(msg) 

#endif

#endif /* !LICENSE_COMMON_H */
